<?php
namespace mysterious\productlayout\controllers;
use function mysterious\productlayout\productLayoutWPB;
class ajax
{
    private static $_instance = null;

    public function __construct()
    {
        add_action('wp_ajax_mst_product_layout_publish', array( $this, 'ajaxPublishTemplate' ));
        add_action('wp_ajax_mst_product_layout_update', array( $this, 'ajaxUpdateTemplate' ));
        add_action('wp_ajax_mst_product_layout_delete', array( $this, 'ajaxDeleteTemplate' ));
        add_action('wp_ajax_mst_product_layout_active', array( $this, 'ajaxActiveTemplate' ));
        add_action('wp_ajax_mst_product_layout_export', array( $this, 'ajaxExportTemplate' ));
        add_action('wp_ajax_mst_product_layout_import', array( $this, 'ajaxImportTemplate' ));
    }

    public static function instance()
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    public function check_nonce()
    {
        $nonce = sanitize_text_field($_POST['nonce']);
        if (!wp_verify_nonce($nonce, 'product-layout-nonce')) {
            wp_send_json_error(array('status' => __('Nonce is invalid',MST_CPLFVC)));
        }
    }
  
    public function ajaxPublishTemplate()
    {
        try {
            $this->check_nonce();
            if(isset($_POST['title']) && isset($_POST['content']) ){
                $title          = isset($_POST['title']) && !empty($_POST['title']) ? sanitize_text_field($_POST['title']) : esc_html__('Template',MST_CPLFVC).' #'.sanitize_text_field($_POST['post_id']);
                $type           = sanitize_text_field($_POST['template_type']);
                $table_name     = productLayoutWPB()->get_table_by_type_template($type);
                productLayoutWPB()->save_template(
                    $table_name,
                    array(
                        'title'         => $title,
                        'images'        => wp_strip_all_tags($_POST['custom_css']),
                        'content'       => stripslashes($_POST['content']) ,
                        'language'      => sanitize_text_field($_POST['language']),
                        'active'        => 0
                    )
                );
                wp_send_json_success(array('mess' => esc_html__('Saved changes',MST_CPLFVC)));
            }
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        } catch (\Exception $ex) {
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        } catch (\Error $ex) {
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        }
       
    }
    
    public function ajaxUpdateTemplate()
    {
        try {
            $this->check_nonce();
            if(isset($_POST['title']) && isset($_POST['content']) && isset($_POST['id'])){
                $title          = isset($_POST['title']) && !empty($_POST['title']) ? sanitize_text_field($_POST['title']) : esc_html__('Template',MST_CPLFVC).' #'.sanitize_text_field($_POST['post_id']);
                $id             = intval( sanitize_text_field($_POST['id']) );
                $type           = sanitize_text_field($_POST['template_type']);
                $table_name     = productLayoutWPB()->get_table_by_type_template($type);
                productLayoutWPB()->update_template(
                    $table_name,
                    array(
                        'title'         => $title,
                        'images'        => wp_strip_all_tags($_POST['custom_css']),
                        'content'       => stripslashes($_POST['content']) 
                    ),
                    array(
                        'id' => $id
                    )
                );
                wp_send_json_success(array('mess' => esc_html__('Saved changes',MST_CPLFVC)));
            }
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        } catch (\Exception $ex) {
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        } catch (\Error $ex) {
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        }
       
    }

    public function ajaxDeleteTemplate()
    {
        try {
            $this->check_nonce();
            if(isset($_POST['id'])){
                $id     = intval( sanitize_text_field($_POST['id']) );
                $type   = sanitize_text_field($_POST['template_type']);
                $table  = productLayoutWPB()->get_table_by_type_template($type);
                productLayoutWPB()->delete_template($table,array('id' => $id));
                wp_send_json_success(array('mess' => esc_html__('Deleted',MST_CPLFVC)));
            }
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        } catch (\Exception $ex) {
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        } catch (\Error $ex) {
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        }
        
    }

    public function ajaxActiveTemplate()
    {
        try {
            $this->check_nonce();
            if(isset($_POST['id'])){
                $type = sanitize_text_field($_POST['template_type']);
                $data = array(
                    'id'        => intval( sanitize_text_field($_POST['id']) ),
                    'language'  => sanitize_text_field($_POST['language'])
                );
                $this->activeThisTemplate( $type, $data );
                
                wp_send_json_success(array('mess' => esc_html__('Saved changes',MST_CPLFVC)));
            }
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        } catch (\Exception $ex) {
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        } catch (\Error $ex) {
            wp_send_json_error(array('mess' => esc_html__('An error occurred. Please try again!!!',MST_CPLFVC)));
        }
        
    }
   
    public function get_all_product_taxonomies($taxonomy='product_cat')
    {
        $args = array(
            'taxonomy'     => $taxonomy,
            'orderby'      => 'name',
            'show_count'   => 0,
            'pad_counts'   => 0,
            'hierarchical' => 1,
            'title_li'     => '',
            'hide_empty'   => 0
        );
        $all_product_categories = get_categories( $args );
        return $all_product_categories;
    }
    
    public function activeThisTemplate( $type, $data = array() )
    {
        $table = productLayoutWPB()->get_table_by_type_template($type);
        $types  = array('category','single','tags','brand'); 
        if( in_array($type,$types) ){
            if($type === 'single'){
                $this->activeProductTemplate( $table, $data );
            }else{
                switch ($type) {
                    case 'category':
                        $data['name']       = 'mst_cplfvc_product_category';
                        $data['taxonomy']   = 'product_cat';
                        break;
                    case 'tags':
                        $data['name']       = 'mst_cplfvc_product_tags';
                        $data['taxonomy']   = 'product_tag';
                        break;
                    default:
                        $data['name']       = 'mst_cplfvc_product_brand';
                        $data['taxonomy']   = 'product_brand';
                        break;
                }
                $this->activeArchiveTemplate( $table, $data );
            }
        }else{
            // SHOP & ATTRIBUTE
            $this->activeOrShopAttributeTemplate( $table, $data );
            if($type === 'attribute'){
                $this->updateOrDeleteAttributeTemplate( $table, $data );
            }
        }
        
    }
    
    public function activeArchiveTemplate( $table, $data = array() )
    {
        $id         = $data['id'];
        $language   = $data['language'];
        $all_product_taxonomies = $this->get_all_product_taxonomies( $data['taxonomy'] );
        $name = !empty($language) ? $data['name'].'_'.$language : $data['name'];
        $templates = productLayoutWPB()->get_all_templates( "SELECT * FROM $table WHERE language = '".$language."'");
        if($id!=0){
            foreach ($all_product_taxonomies as $key => $product_archive){
                $id_by_term = get_term_meta( $product_archive->term_taxonomy_id, $name, true );
                if( $id_by_term==='active' || empty($id_by_term) ){
                    update_term_meta( $product_archive->term_taxonomy_id, $name, 'active' ); 
                }
            }
            foreach ($templates as $key => $row) {
                if( $row->id != $id ){
                    $active     = 0;
                    $id_where   = $row->id;
                }else{
                    $active     = 1;
                    $id_where   = $id;
                }
                productLayoutWPB()->update_template(
                    $table,
                    array('active' => $active),
                    array('id' => $id_where)
                );
            }
        }else{    
            foreach ($all_product_taxonomies as $key => $product_archive){
                if( get_term_meta ($product_archive->term_taxonomy_id,  $name, true ) === 'active' ){
                    update_term_meta($product_archive->term_taxonomy_id, $name, '' );
                }
            }
            foreach ($templates as $key => $row) {
                productLayoutWPB()->update_template( $table, array('active' => 0), array('id' => $row->id) );
            }
        }
    }

    public function activeOrShopAttributeTemplate( $table, $data = array() )
    {
        $id = $data['id'];
        $language = $data['language'];
        $templates = productLayoutWPB()->get_all_templates( "SELECT * FROM $table WHERE language = '".$language."'");
        if($id!=0){
            foreach ($templates as $key => $row) {
                if( $row->id != $id ){
                    $active     = 0;
                    $id_where   = $row->id;
                }else{
                    $active     = 1;
                    $id_where   = $id;
                }
                productLayoutWPB()->update_template(
                    $table,
                    array('active' => $active),
                    array('id' => $id_where)
                );
            }
        }else{
            foreach ($templates as $key => $row) {
                productLayoutWPB()->update_template( $table, array('active' => 0), array('id' => $row->id) );
            }
        }
        
    }

    public function activeProductTemplate( $table, $data = array() )
    {
        $id = $data['id'];
        $language = $data['language'];
        $all_product_categories = $this->get_all_product_taxonomies('product_cat');
        $name = !empty($language) ? 'mst_pclfvc_product_cat_single_'.$language : 'mst_pclfvc_product_cat_single';
        $templates = productLayoutWPB()->get_all_templates( "SELECT * FROM $table WHERE language = '".$language."'");
        if($id!=0){
            
            foreach ($all_product_categories as $key => $product_cat){
                $temlate_by_term = get_term_meta( $product_cat->term_id, $name, true );
                if( $temlate_by_term === 'active' || empty($temlate_by_term) ){
                    update_term_meta( $product_cat->term_id, $name, 'active' ) ;
                } 
            }
            
            foreach ($templates as $key => $row) {
                if( $row->id != $id ){
                    $active     = 0;
                    $id_where   = $row->id;
                }else{
                    $active     = 1;
                    $id_where   = $id;
                }
                productLayoutWPB()->update_template(
                    $table,
                    array('active' => $active),
                    array('id' => $id_where)
                );
            }
            
        }else{
            $name_single = !empty($language) ? 'mst_pclfvc_product_single_'.$language : 'mst_pclfvc_product_single';
            foreach ($all_product_categories as $key => $cat){
                if( get_term_meta( $cat->term_id, $name, true ) === 'active' ){
                    delete_term_meta($cat->term_id, $name);
                }
            }
            // set all product templates with active column is 0
            foreach ($templates as $key => $row) {
                productLayoutWPB()->update_template( $table, array('active' => 0), array('id' => $row->id) );
            }
        }
    }

    public function updateOrDeleteAttributeTemplate( $table, $data = array() )
    {
        
        $id         = $data['id'];
        $language   = $data['language'];
        $all_attributes = wc_get_attribute_taxonomies();  // get all product attributes;
        $name       = 'mst_template_attribute';
        
        if($id!=0){
            foreach($all_attributes as $key => $attribute){
                $attribute_id   = $attribute->attribute_id;
                $option_name    = !empty($language) ? $name.'_'.$attribute_id.'_'.$language : $name.'_'.$attribute_id;
                $template       = get_option($option_name);
                if( $template ==='active' || empty($template) ){
                    update_option($option_name, 'active'); 
                }
            }
        }else{    
            foreach($all_attributes as $key => $attribute){
                $attribute_id   = $attribute->attribute_id;
                $option_name    = !empty($language) ? $name.'_'.$attribute_id.'_'.$language : $name.'_'.$attribute_id;
                $template       = get_option($option_name);
                if( $template ==='active' ){
                    delete_option($option_name); 
                }
            }
           
        }
    }

    public function ajaxExportTemplate()
    {
        try{
            $this->check_nonce();
            wp_send_json_success();
        } catch (\Exception $ex) {
            wp_send_json_error(array('mess' => __('An error occurred. Please try again!!!',MST_CPLFVC)));
        } catch (\Error $ex) {
            wp_send_json_error(array('mess' => __('An error occurred. Please try again!!!',MST_CPLFVC)));
        }
    }

    public function ajaxImportTemplate()
    {
        try {
            $this->check_nonce();
            if (isset($_FILES['file']['type'])) {
                if ($_FILES['file']['type'] == "application/json") {
                    if (!empty($_FILES['file']['tmp_name'])) {
                        $fileJson = $_FILES['file']['tmp_name'];
                        $data = file_get_contents($fileJson);
                        $data = json_decode($data, true);
                        $flag = false;
                        $types = array("shop","category","single","tags");
                        foreach($types as $type){
                            if(isset($data[$type]) && count($data[$type]) > 0){
                                foreach ($data[$type] as $key => $value) {
                                    if (isset($value['title']) && isset($value['content']) && isset($value['images']) && isset($value['language']) && isset($value['active']) ) {
                                        $table = productLayoutWPB()->get_table_by_type_template($type);
                                        $insert = productLayoutWPB()->save_template(
                                                    $table,
                                                    array( 
                                                        'title' => $value['title'],
                                                        'images'=> $value['images'],
                                                        'content' => $value['content'] ,
                                                        'language' => $value['language'],
                                                        'active' => $value['active']
                                                    )
                                        );
                                        $flag = true;
                                    }
                                }
                            }
                        }
                        if (!$flag) {
                            wp_send_json_error(array('mess' => __('Import Failed.',MST_CPLFVC)));
                        }
                        wp_send_json_success(array('mess' => __('Imported successfully.',MST_CPLFVC)));
                    } else {
                        wp_send_json_error(array('mess' => __('File not found.',MST_CPLFVC)));
                    }
                } else {
                    wp_send_json_error(array('mess' => __('File not correct format.',MST_CPLFVC)));
                }
            }
            wp_send_json_error(array('mess' => __('Please upload 1 file to import.',MST_CPLFVC)));         
        } catch (\Exception $ex) {
            wp_send_json_error();
        } catch (\Error $ex) {
            wp_send_json_error();
        }
    }
}